import sys
import typing


def bake_all():
    ''' Bake Entire Fluid Simulation

    '''

    pass


def bake_data():
    ''' Bake Fluid Data

    '''

    pass


def bake_guides():
    ''' Bake Fluid Guiding

    '''

    pass


def bake_mesh():
    ''' Bake Fluid Mesh

    '''

    pass


def bake_noise():
    ''' Bake Fluid Noise

    '''

    pass


def bake_particles():
    ''' Bake Fluid Particles

    '''

    pass


def free_all():
    ''' Free Entire Fluid Simulation

    '''

    pass


def free_data():
    ''' Free Fluid Data

    '''

    pass


def free_guides():
    ''' Free Fluid Guiding

    '''

    pass


def free_mesh():
    ''' Free Fluid Mesh

    '''

    pass


def free_noise():
    ''' Free Fluid Noise

    '''

    pass


def free_particles():
    ''' Free Fluid Particles

    '''

    pass


def pause_bake():
    ''' Pause Bake

    '''

    pass


def preset_add(name: str = "",
               remove_name: bool = False,
               remove_active: bool = False):
    ''' Add or remove a Fluid Preset

    :param name: Name, Name of the preset, used to make the path name
    :type name: str
    :param remove_name: remove_name
    :type remove_name: bool
    :param remove_active: remove_active
    :type remove_active: bool
    '''

    pass
